/*
 * DeterministicLL1Grammar.cpp
 *
 *  Created on: 9. 6. 2015
 *	  Author: Jan Travnicek
 */

#include "DeterministicLL1Grammar.h"
#include "LL1ParseTable.h"
#include "HandleFirstFirstConflict.h"
#include "HandleFirstFollowConflict.h"

#include <grammar/ContextFree/CFG.h>
#include <exception/AlibException.h>
#include <algorithm>

namespace grammar {

namespace parsing {

bool transformToLL1 ( grammar::CFG & grammar, const alphabet::Symbol & terminal, const alphabet::Symbol & nonterminal, const std::set < std::vector < alphabet::Symbol > > & rhsds ) {
	if ( std::any_of ( rhsds.begin ( ), rhsds.end ( ), [] ( const std::vector < alphabet::Symbol > & rhs ) {
				return rhs.size ( ) == 0;
			} ) )
		return HandleFirstFollowConflict::handleFirstFollowConflict ( grammar, terminal, nonterminal, rhsds );
	else
		return HandleFirstFirstConflict::handleFirstFirstConflict ( grammar, terminal, nonterminal, rhsds );
}

grammar::CFG DeterministicLL1Grammar::convert ( const grammar::CFG & param ) {
	grammar::CFG grammar = param;

	while ( true ) {
		std::map < std::pair < std::variant < alphabet::Symbol, string::Epsilon >, alphabet::Symbol >, std::set < std::vector < alphabet::Symbol > > > parseTable = LL1ParseTable::parseTable ( grammar );

		bool deterministic	= true;
		bool cantTransform	= true;
		bool errorInEpsilon = false;

		for ( const std::pair < std::pair < std::variant < alphabet::Symbol, string::Epsilon >, alphabet::Symbol >, std::set < std::vector < alphabet::Symbol > > > & elem : parseTable )
			if ( elem.second.size ( ) > 1 ) {
				if ( elem.first.first.is < string::Epsilon > ( ) ) {
					errorInEpsilon = true;
					continue;
				}

				deterministic = false;

				if ( transformToLL1 ( grammar, elem.first.first.get < alphabet::Symbol > ( ), elem.first.second, elem.second ) ) {
					cantTransform = false;
					break;
				}
			}

		if ( deterministic )
			return grammar;

		if ( cantTransform ) {
			if ( errorInEpsilon )
				throw exception::AlibException ( "Cant handle conflict in epsilon" );
			else
				throw exception::AlibException ( "Unable to find transformable pattern in rules of the grammar" );
		}
	}
}

auto DeterministicLL1GrammarCFG = DeterministicLL1Grammar::RegistratorWrapper < grammar::CFG, grammar::CFG > ( DeterministicLL1Grammar::getInstance ( ), DeterministicLL1Grammar::convert );

grammar::Grammar DeterministicLL1Grammar::convert ( const grammar::Grammar & grammar ) {
	return getInstance ( ).dispatch ( grammar.getData ( ) );
}

} /* namespace parsing */

} /* namespace grammar */
