/*
 * alphabetManip.cpp
 *
 *  Created on: 26. 3. 2014
 *	  Author: Jan Travnicek
 */

#include <tclap/CmdLine.h>
#include <common/GlobalData.h>
#include <vector>

#include <factory/XmlDataFactory.hpp>
#include <exception/AlibException.h>
#include <tree/RankedTreeWrapper.h>
#include <tree/UnrankedTreeWrapper.h>
#include <string/String.h>
#include <container/ObjectsSet.h>

#include <iostream>

template < class T >
void process ( T & data, TCLAP::ValueArg < std::string > & operation, std::deque < sax::Token > & argumentTokens ) {
	typedef typename std::remove_const < typename std::remove_reference < decltype ( data.getAlphabet ( ) ) >::type >::type alphabetType;

	if ( operation.getValue ( ) == "get" ) {
		std::chrono::measurements::end ( );
		std::chrono::measurements::start ( "Algorithm", std::chrono::measurements::Type::MAIN );

		const alphabetType & res = data.getAlphabet ( );

		std::chrono::measurements::end ( );
		std::chrono::measurements::start ( "Output write", std::chrono::measurements::Type::AUXILARY );

		alib::XmlDataFactory::toStdout ( res );
	} else if ( operation.getValue ( ) == "add" ) {
		alphabetType symbols = alib::XmlDataFactory::fromTokens < alphabetType > ( argumentTokens );

		std::chrono::measurements::end ( );
		std::chrono::measurements::start ( "Algorithm", std::chrono::measurements::Type::MAIN );

		data.extendAlphabet ( symbols );

		std::chrono::measurements::end ( );
		std::chrono::measurements::start ( "Output write", std::chrono::measurements::Type::AUXILARY );

		alib::XmlDataFactory::toStdout ( data );
	} else if ( operation.getValue ( ) == "set" ) {
	} else if ( operation.getValue ( ) == "remove" ) {
	} else {
		throw exception::AlibException ( "Invalid operation" );
	}
}

int main ( int argc, char * argv[] ) {
	try {
		TCLAP::CmdLine cmd ( "Alphabet manipulation binary", ' ', "0.01" );

		std::vector < std::string > allowed;
		allowed.push_back ( "get" );
		allowed.push_back ( "set" );
		allowed.push_back ( "add" );
		allowed.push_back ( "remove" );
		TCLAP::ValuesConstraint < std::string > allowedVals ( allowed );

		TCLAP::ValueArg < std::string > operation ( "o", "operation", "Operation type", false, "", & allowedVals );
		cmd.add ( operation );

		TCLAP::ValueArg < std::string > input ( "i", "input", "Input from file", false, "-", "file" );
		cmd.add ( input );

		TCLAP::ValueArg < std::string > argument ( "a", "argument", "Operation argument", false, "", "file" );
		cmd.add ( argument );

		TCLAP::SwitchArg measure ( "m", "measure", "Measure times", false );
		cmd.add ( measure );

		TCLAP::SwitchArg verbose ( "v", "verbose", "Be verbose", false );
		cmd.add ( verbose );

		cmd.parse ( argc, argv );

		if(verbose.isSet())
			common::GlobalData::verbose = true;
		if(measure.isSet())
			common::GlobalData::measure = true;

		std::chrono::measurements::start ( "Overal", std::chrono::measurements::Type::OVERALL );
		std::chrono::measurements::start ( "Input read", std::chrono::measurements::Type::AUXILARY );

		std::deque < sax::Token > inputTokens;

		if ( input.isSet ( ) ) {
			if ( input.getValue ( ) == "-" )
				sax::SaxParseInterface::parseStdin ( inputTokens );
			else
				sax::SaxParseInterface::parseFile ( input.getValue ( ), inputTokens );
		} else {
			sax::SaxParseInterface::parseStdin ( inputTokens );
		}

		std::deque < sax::Token > argumentTokens;

		if ( argument.isSet ( ) ) {
			if ( argument.getValue ( ) == "-" )
				sax::SaxParseInterface::parseStdin ( argumentTokens );
			else
				sax::SaxParseInterface::parseFile ( argument.getValue ( ), argumentTokens );
		}

		if ( alib::XmlDataFactory::first < tree::RankedTreeWrapper > ( inputTokens ) ) {
			tree::RankedTreeWrapper data = alib::XmlDataFactory::fromTokens < tree::RankedTreeWrapper > ( inputTokens );
			process ( data, operation, argumentTokens );
		} else if ( alib::XmlDataFactory::first < tree::UnrankedTreeWrapper > ( inputTokens ) ) {
			tree::UnrankedTreeWrapper data = alib::XmlDataFactory::fromTokens < tree::UnrankedTreeWrapper > ( inputTokens );
			process ( data, operation, argumentTokens );
		} else if ( alib::XmlDataFactory::first < string::String > ( inputTokens ) ) {
			string::String data = alib::XmlDataFactory::fromTokens < string::String > ( inputTokens );
			process ( data, operation, argumentTokens );
		} else {
			throw exception::AlibException ( "Invalid data type" );
		}

		std::chrono::measurements::end ( );
		std::chrono::measurements::end ( );

		if ( measure.getValue ( ) ) std::clog << std::chrono::measurements::results ( ) << std::endl;

		return 0;
	} catch ( const exception::AlibException & exception ) {
		alib::XmlDataFactory::toStdout ( exception );
		return 1;
	} catch ( const TCLAP::ArgException & exception ) {
		std::cout << exception.error ( ) << std::endl;
		return 2;
	} catch ( const std::exception & exception ) {
		std::cerr << "Exception caught: " << exception.what ( ) << std::endl;
		return 3;
	} catch ( ... ) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
